/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.RutaCombinacionDAO;
import com.rjconsultores.ventaboletos.dao.TramoDAO;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.entidad.RutaCombinacion;
import com.rjconsultores.ventaboletos.entidad.Tramo;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

/**
 *
 * @author Rafius
 */
@Repository("rutaCombinacionDAO")
public class RutaCombinacionHibernateDAO extends GenericHibernateDAO<RutaCombinacion, Integer>
        implements RutaCombinacionDAO {

	private static Logger log = LoggerFactory.getLogger(RutaCombinacionHibernateDAO.class);
	
    @Autowired
    private TramoDAO tramoDAO;
    
    @Autowired
    public RutaCombinacionHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
        setSessionFactory(factory);
    }

    public boolean busquedaRutaTramo(Ruta ruta, Tramo tramo, Boolean ativo) {

        Criteria c = getSession().createCriteria(getPersistentClass());

        if (ativo) {
            c.add(Restrictions.eq("activo", Boolean.TRUE));
        } else {
            c.add(Restrictions.eq("activo", Boolean.FALSE));
        }
        
        c.setProjection(Projections.rowCount());

        c.add(Restrictions.eq("ruta", ruta));
        c.add(Restrictions.eq("tramo", tramo));
        
        return HibernateFix.count(c.list()) > 0;
    }

    public RutaCombinacion busquedaTramoRutaTramo(Ruta ruta, Tramo trBuscaOrigemDestino) {

        String hql = " select rc from RutaCombinacion rc, Tramo t "
                + " where rc.tramo.tramoId = t.tramoId and rc.activo = 0 "
                + " and rc.ruta.rutaId = " + ruta.getRutaId()
                + " and t.origem.paradaId = " + trBuscaOrigemDestino.getOrigem().getParadaId()
                + " and t.destino.paradaId = " + trBuscaOrigemDestino.getDestino().getParadaId();
        Query sq = getSession().createQuery(hql);

        RutaCombinacion rc = new RutaCombinacion();
        if (sq.list().size() > 0) {
            rc = (RutaCombinacion) sq.list().get(sq.list().size() - 1);
        }

        return rc;
    }

    public Tramo busqueda(Ruta ruta, Parada origen, Parada destino) {
        Criteria c = this.makeCriteria();
        c.add(Restrictions.eq("ruta", ruta));
        c.add(Restrictions.eq("activo", Boolean.TRUE));

        Criteria c2 = c.createCriteria("tramo");
        c2.add(Restrictions.eq("origem", origen));
        c2.add(Restrictions.eq("destino", destino));
        c2.add(Restrictions.eq("activo", Boolean.TRUE));

        List<RutaCombinacion> list = c.list();

        if (list.size() != 1) {
            return null;
        }
        return list.get(0).getTramo();
    }

    public RutaCombinacion busquedaTramoRutaOrigemDestino(Ruta ruta, Parada origem, Parada destino) {
        Criteria c = this.makeCriteria();
        c.add(Restrictions.eq("ruta", ruta));

        Criteria c2 = c.createCriteria("tramo");
        c2.add(Restrictions.eq("origem", origem));
        c2.add(Restrictions.eq("destino", destino));
        c2.add(Restrictions.eq("activo", Boolean.TRUE));

        if (c.list().size() > 0) {
            return (RutaCombinacion) c.list().get(0);
        }

        return null;

    }

    public List<RutaCombinacion> obtenerPorRuta(Ruta ruta) {
        Criteria c = this.makeCriteria();
        c.add(Restrictions.eq("ruta", ruta));
        c.add(Restrictions.eq("activo", Boolean.TRUE));

        return c.list();
    }

    public boolean buscarRutaCombinacionExisteTramo(Tramo tramo) {
        Criteria c = this.makeCriteria();
        c.add(Restrictions.eq("tramo", tramo));
        c.add(Restrictions.eq("activo", Boolean.TRUE));

        if (c.list().isEmpty()) {
            return false;
        } else {
            return true;
        }


    }
    @Override
    public List<Tramo> tramosFaltantes(Integer rutaId){
    	Session session = getSession();
    	List<Tramo> lsTramoFaltantes = new ArrayList<Tramo>();
    	
    	StringBuilder sb = new StringBuilder("");
    	sb.append("  select t.origen_id, ");
    	sb.append("         rs.numsecuencia  ");
    	sb.append(" from ruta_secuencia rs ");
    	sb.append("   inner join tramo t on t.tramo_id = rs.tramo_id ");
    	sb.append(" where rs.activo = 1 and ");
    	sb.append("    t.activo = 1 and ");
    	sb.append("    rs.ruta_id = :rutaId ");
    	sb.append(" union ");
    	sb.append(" select t.destino_id, ");
    	sb.append("         rs.numsecuencia  ");
    	sb.append(" from ruta_secuencia rs ");
    	sb.append("   inner join tramo t on t.tramo_id = rs.tramo_id ");
    	sb.append(" where rs.activo = 1 and ");
    	sb.append("    t.activo = 1 and ");
    	sb.append("    rs.ruta_id = :rutaId and ");
    	sb.append("    rs.numsecuencia = ");
    	sb.append("    (select max(rs.numsecuencia) from ");
    	sb.append("     ruta_secuencia rs inner join tramo t on ");
    	sb.append("      t.tramo_id = rs.tramo_id where rs.activo = ");
    	sb.append("       1 and t.activo = 1 and rs.ruta_id = :rutaId ) ");
    	sb.append(" order by 2 ");
    	
    	
    	SQLQuery sqlQuery = session.createSQLQuery(sb.toString());
    	sqlQuery.setInteger("rutaId", rutaId);
    	
    	List<Object[]> lsParadas = sqlQuery.list();
    	Integer origenId = null;
    	Integer destinoId = null;
    	for(int i = 0 ; i < lsParadas.size();i++){
    		origenId = ((BigDecimal)lsParadas.get(i)[0]).intValue();
    		for(int j = i+1 ; j < lsParadas.size();j++){
    			destinoId = ((BigDecimal)lsParadas.get(j)[0]).intValue();
    			
    			log.debug(origenId + " - " + destinoId);
    			
    			sb = new StringBuilder();
    			sb.append(" select count(*) ");
    			sb.append(" from ruta_combinacion rc ");
    			sb.append(" 	inner join tramo t on t.tramo_id = rc.tramo_id ");    			
    			sb.append(" where  ");
    			sb.append(" 	rc.ruta_id = :rutaId ");
    			sb.append(" 	and rc.activo = 1 ");
    			sb.append(" 	and t.activo = 1 ");
    			sb.append(" 	and t.origen_id = :origenId ");
    			sb.append(" 	and t.destino_id = :destinoId ");
    			
    			
    			SQLQuery sqlQueryExists = session.createSQLQuery(sb.toString());
    			sqlQueryExists.setInteger("rutaId", rutaId);
    			sqlQueryExists.setInteger("origenId", origenId);
    			sqlQueryExists.setInteger("destinoId", destinoId);
    			
    			Long count = HibernateFix.count(sqlQueryExists.list());
    			
    			if (count == 0){
    				log.info("No existe combinacion " + origenId + " - " + destinoId);
    				
    				
    				Parada paradaOrigem = new Parada();
    				paradaOrigem.setParadaId(origenId);
    				
    				Parada paradaDestino = new Parada();
    				paradaDestino.setParadaId(destinoId);
    				
    				List<Tramo> list = tramoDAO.obtenerListPorOrigemDestino(paradaOrigem, paradaDestino);
    				
    				if (list.size() == 0){
    					log.info("No existe tramo " + origenId + " - " + destinoId);
    				}else{
    					lsTramoFaltantes.add(list.get(0));
    				}
    				
    			}
    		}
    		
    	}
    	
    	return lsTramoFaltantes;
    }
    
    public int borrarCombinaciones(Ruta ruta){
    	Query query = this.getSession().createQuery("UPDATE RutaCombinacion set activo = false, fecmodif =  current_timestamp(), usuarioId= :usuario where ruta = :ruta");
    	query.setParameter("ruta", ruta);
    	query.setParameter("usuario", UsuarioLogado.getUsuarioLogado().getUsuarioId());
    	
    	return query.executeUpdate();
    }
    
    public void activar(Ruta ruta, Tramo trBuscaOrigemDestino){
    	String sql = " select rc.rutacombinacionId from RutaCombinacion rc, Tramo t "
        + " where rc.tramo.tramoId = t.tramoId and rc.activo = 0 "
        + " and rc.ruta.rutaId = " + ruta.getRutaId()
        + " and t.origem.paradaId = " + trBuscaOrigemDestino.getOrigem().getParadaId()
        + " and t.destino.paradaId = " + trBuscaOrigemDestino.getDestino().getParadaId();
    	
    	getSession().createQuery("update RutaCombinacion set activo = true where rutacombinacionId in ("+ sql+" )").executeUpdate();
    }
}
