/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import com.rjconsultores.ventaboletos.dao.TramoDAO;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Tramo;
import com.rjconsultores.ventaboletos.entidad.Via;
import java.util.List;
import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

/**
 * 
 * @author Administrador
 */
@Repository("tramoDAO")
public class TramoHibernateDAO extends GenericHibernateDAO<Tramo, Integer>
		implements TramoDAO {

	@Autowired
	public TramoHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<Tramo> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.addOrder(Order.asc("desctramo"));

		return c.list();
	}

	public Tramo obtenerPorOrigemDestino(Parada origem, Parada destino) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		Tramo t = new Tramo();
		if (c.list().size() > 0) {
			t = (Tramo) c.list().get(0);
		}

		return t;
	}

	public Tramo obtenerTramotPorOrigemDestinoVia(Parada origem, Parada destino, Via via) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("via", via));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return (Tramo) c.uniqueResult();
	}

	public List<Tramo> obtenerListPorOrigemDestino(Parada origem, Parada destino) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.addOrder(Order.asc("desctramo"));

		return c.list();
	}

	public List<Tramo> obtenerPorOrigemDestinoFetchLazy(Parada origem, Parada destino) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.addOrder(Order.asc("desctramo"));

		c.setFetchMode("origem", FetchMode.LAZY);
		c.setFetchMode("destino", FetchMode.LAZY);
		c.setFetchMode("lsTramoTiempo", FetchMode.LAZY);
		c.setFetchMode("rutaSecuenciaList", FetchMode.LAZY);
		c.setFetchMode("rutaCombinacionList", FetchMode.LAZY);
		c.setFetchMode("tramoServicioList", FetchMode.LAZY);
		c.setFetchMode("tramoKms", FetchMode.LAZY);

		return c.list();
	}

	// Obtener Vias Possiveis
	public List<Via> obtenerViasOrigemDestino(Parada origem, Parada destino) {
		Criteria c = makeCriteria();
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.addOrder(Order.asc("desctramo"));

		c.setProjection(Projections.property("via"));

		return c.list();
	}

	public Tramo busca(Parada origem, Parada destino, Via via) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("activo", Boolean.TRUE));
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("via", via));

		return (Tramo) c.uniqueResult();
	}

	public List<Tramo> buscarCveParada(String origen, String destino, String via) {
		Criteria c = getSession().createCriteria(getPersistentClass());

		c.createCriteria("origem").add(Restrictions.eq("paradaId", Integer.parseInt(origen)));
		c.createCriteria("destino").add(Restrictions.eq("paradaId", Integer.parseInt(destino)));
		c.createCriteria("via").add(Restrictions.eq("viaId", Integer.parseInt(via)));

		c.add(Restrictions.eq("activo", Boolean.TRUE));

		return c.list();
	}

	public Tramo obtenerTramoUnicoRuta(Parada origem, Parada destino, Via via,
			ClaseServicio claseServicio) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("origem", origem));
		c.add(Restrictions.eq("destino", destino));
		c.add(Restrictions.eq("via", via));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.setFetchMode("origem", FetchMode.LAZY);
		c.setFetchMode("destino", FetchMode.LAZY);
		c.setFetchMode("lsTramoTiempo", FetchMode.LAZY);
		c.setFetchMode("rutaSecuenciaList", FetchMode.LAZY);
		c.setFetchMode("rutaCombinacionList", FetchMode.LAZY);
		c.setFetchMode("tramoServicioList", FetchMode.LAZY);
		c.setFetchMode("tramoKms", FetchMode.LAZY);

		Criteria c2 = c.createCriteria("tramoServicioList");
		c2.add(Restrictions.eq("claseServicio", claseServicio));
		c2.add(Restrictions.eq("activo", Boolean.TRUE));

		c.setMaxResults(1);
		List<Tramo> result = c.list();

		if (result.isEmpty()) {
			return null;
		}

		return result.get(0);
	}

	public Long count(Via via) {
		Criteria c = this.makeCriteria();

		c.add(Restrictions.eq("via", via));
		c.add(Restrictions.eq("activo", Boolean.TRUE));

		c.setProjection(Projections.rowCount());

		return HibernateFix.count(c.list());
	}
}
